from nose.plugins.skip import SkipTest

from tqdm import tqdm
from tests_tqdm import with_setup, pretest, posttest, StringIO, closing


@with_setup(pretest, posttest)
def test_pandas():
    """ Test pandas.DataFrame.groupby(0).progress_apply """
    try:
        from numpy.random import randint
        from tqdm import tqdm_pandas
        import pandas as pd
    except:
        raise SkipTest

    with closing(StringIO()) as our_file:
        df = pd.DataFrame(randint(0, 100, (1000, 6)))
        tqdm_pandas(tqdm(file=our_file, leave=False, ascii=True))
        df.groupby(0).progress_apply(lambda x: None)

        our_file.seek(0)

        # don't expect final output since no `leave` and
        # high dynamic `miniters`
        nexres = '100%|##########| 101/101'
        if nexres in our_file.read():
            our_file.seek(0)
            raise AssertionError("\nDid not expect:\n{0}\nIn:{1}\n".format(
                nexres, our_file.read()))


@with_setup(pretest, posttest)
def test_pandas_leave():
    """ Test pandas with `leave=True` """
    try:
        from numpy.random import randint
        from tqdm import tqdm_pandas
        import pandas as pd
    except:
        raise SkipTest

    with closing(StringIO()) as our_file:
        df = pd.DataFrame(randint(0, 100, (1000, 6)))
        tqdm_pandas(tqdm(file=our_file, leave=True, ascii=True))
        df.groupby(0).progress_apply(lambda x: None)

        our_file.seek(0)

        exres = '100%|##########| 101/101'
        if exres not in our_file.read():
            our_file.seek(0)
            raise AssertionError("\nExpected:\n{0}\nIn:{1}\n".format(
                exres, our_file.read()))



from ._tqdm import tqdm
from ._version import __version__  # NOQA
from docopt import docopt
import sys
import re
__all__ = ["main"]


def cast(val, typ):
    if val == 'None':
        return None
    if typ == 'bool':
        return str(val) == 'True'
    # try:
    return eval(typ + '("' + str(val) + '")')
    # except:
    #     # print val, typ
    #     if val == 'special':
    #         return 'whatever... just an example'
    #     else:
    #         return eval(typ + '()')


RE_OPTS = re.compile(r' {8}(\w+)\s{2,}:\s*(str|int|float|bool)', flags=re.M)
# RE_OPTS_SOME = re.compile(r' {8}(\w+)  : (str|int|float)', flags=re.M)
# RE_OPTS_BOOL = re.compile(r' {8}(\w+)  : bool', flags=re.M)


def main():
    d = tqdm.__init__.__doc__
    opt_types = dict(RE_OPTS.findall(d))
    # d = RE_OPTS_SOME.sub(r'  --\1=<v>  ', d)
    # d = RE_OPTS_BOOL.sub(r'  --\1      ', d)
    d = RE_OPTS.sub(r'  --\1=<v>  : \2', d)
    d = d[d.find('  --desc='):d.find('Returns\n')]
    __doc__ = """
Usage:
    tqdm [--help | options]

Options:
  -h, --help     Print this help and exit
  -v, --version  Print version and exit
""" + d
    opts = docopt(__doc__, version=__version__)

    try:
        for opt in opt_types:
            opt_types[opt] = cast(opts['--' + opt], opt_types[opt])
        for i in tqdm(sys.stdin, **opt_types):
            sys.stdout.write(i)
    except:  # pragma: no cover
        sys.stderr.write(__doc__ + '\n')
        for i in sys.stdin:
            sys.stdout.write(i)



import sys
import subprocess
from tqdm import main
from docopt import DocoptExit
from copy import deepcopy

from tests_tqdm import with_setup, pretest, posttest, _range


# WARNING: this should be the last test as it messes with sys.stdin, argv
@with_setup(pretest, posttest)
def test_main():
    """ Test command line pipes """
    ls_out = subprocess.Popen(('ls'), stdout=subprocess.PIPE).communicate()[0]
    ls = subprocess.Popen(('ls'), stdout=subprocess.PIPE)
    res = subprocess.Popen(('python', '-c', 'from tqdm import main; main()'),
                           stdout=subprocess.PIPE,
                           stdin=ls.stdout,
                           stderr=subprocess.STDOUT).communicate()[0]
    ls.wait()

    # actual test:

    assert (ls_out in res)

    # semi-fake test which gets coverage:
    try:
        _SYS = (deepcopy(sys.stdin), deepcopy(sys.argv))
    except:
        pass

    sys.stdin = map(str, _range(int(1e3)))
    sys.argv = ['', '--desc', 'Test command line pipes',
                '--ascii', 'True', '--unit_scale', 'True']
    import tqdm.__main__  # NOQA

    sys.argv = ['', '--bad', 'arg',
                '--ascii', 'True', '--unit_scale', 'True']
    try:
        main()
    except DocoptExit as e:
        if """Usage:
    tqdm [--help | options]""" not in str(e):
            raise

    try:
        sys.stdin, sys.argv = _SYS
    except:
        pass
